@@ -2,7 +2,7 @@
 # FlashGBX
 # Author: Lesserkuma (github.com/lesserkuma)
 
-import hashlib, re, zlib, string, os, json
+import hashlib, re, zlib, string, os, json, copy
 from . import Util
 
 class RomFileAGB:
@@ -102,6 +102,14 @@ class RomFileAGB:
 		if (data["game_title"] == "NGC-HIKARU3" and data["game_code"] == "GHTJ" and data["header_checksum"] == 0xB3):
 			data["dacs_8m"] = True
 		
+		# e-Reader
+		data["ereader"] = False
+		if (data["game_title"] == "CARDE READER" and data["game_code"] == "PEAJ" and data["header_checksum"] == 0x9E) or \
+		(data["game_title"] == "CARDEREADER+" and data["game_code"] == "PSAJ" and data["header_checksum"] == 0x85) or \
+		(data["game_title"] == "CARDE READER" and data["game_code"] == "PSAE" and data["header_checksum"] == 0x95):
+			data["ereader"] = True
+
+		data["unchanged"] = copy.copy(data)
 		self.DATA = data
 		data["db"] = self.GetDatabaseEntry()
 		return data
@@ -115,6 +123,16 @@ class RomFileAGB:
 				db = json.loads(db)
 				if data["header_sha1"] in db.keys():
 					db_entry = db[data["header_sha1"]]
+					if db_entry["gc"] in ("ZMAJ", "ZMBJ", "ZMDE"):
+						db_entry["gc"] = "AGS-{:s}".format(db_entry["gc"])
+					elif db_entry["gc"] == "ZBBJ":
+						db_entry["gc"] = "NTR-{:s}".format(db_entry["gc"])
+					elif db_entry["gc"] == "PEAJ":
+						db_entry["gc"] = "PEC-{:s}".format(db_entry["gc"])
+					elif db_entry["gc"] in ("PSAJ", "PSAE"):
+						db_entry["gc"] = "PES-{:s}".format(db_entry["gc"])
+					else:
+						db_entry["gc"] = "AGB-{:s}".format(db_entry["gc"])
 		else:
 			print("FAIL: Database for Game Boy Advance titles not found at {0:s}/db_AGB.json".format(Util.CONFIG_PATH))
 		return db_entry
